@@ -61,6 +61,7 @@ class ScheduledMessages;
 class CloudThemes;
 class Streaming;
 class MediaRotation;
+class Histories;
 
 class Session final {
 public:
@@ -96,6 +97,9 @@ public:
 	[[nodiscard]] MediaRotation &mediaRotation() const {
 		return *_mediaRotation;
 	}
+	[[nodiscard]] Histories &histories() const {
+		return *_histories;
+	}
 	[[nodiscard]] MsgId nextNonHistoryEntryId() {
 		return ++_nonHistoryEntryId;
 	}
@@ -367,22 +371,8 @@ public:
 		const Dialogs::Key &key1,
 		const Dialogs::Key &key2);
 
-	template <typename ...Args>
-	not_null<HistoryMessage*> makeMessage(Args &&...args) {
-		return static_cast<HistoryMessage*>(
-			registerMessage(
-				std::make_unique<HistoryMessage>(
-					std::forward<Args>(args)...)));
-	}
-
-	template <typename ...Args>
-	not_null<HistoryService*> makeServiceMessage(Args &&...args) {
-		return static_cast<HistoryService*>(
-			registerMessage(
-				std::make_unique<HistoryService>(
-					std::forward<Args>(args)...)));
-	}
-	void destroyMessage(not_null<HistoryItem*> item);
+	void registerMessage(not_null<HistoryItem*> item);
+	void unregisterMessage(not_null<HistoryItem*> item);
 
 	// Returns true if item found and it is not detached.
 	bool checkEntitiesAndViewsUpdate(const MTPDmessage &data);
@@ -694,7 +684,7 @@ public:
 	void clearLocalStorage();
 
 private:
-	using Messages = std::unordered_map<MsgId, std::unique_ptr<HistoryItem>>;
+	using Messages = std::unordered_map<MsgId, not_null<HistoryItem*>>;
 
 	void suggestStartExport();
 
@@ -715,7 +705,8 @@ private:
 
 	const Messages *messagesList(ChannelId channelId) const;
 	not_null<Messages*> messagesListForInsert(ChannelId channelId);
-	HistoryItem *registerMessage(std::unique_ptr<HistoryItem> item);
+	not_null<HistoryItem*> registerMessage(
+		std::unique_ptr<HistoryItem> item);
 	void changeMessageId(ChannelId channel, MsgId wasId, MsgId nowId);
 	void removeDependencyMessage(not_null<HistoryItem*> item);
 
@@ -968,7 +959,6 @@ private:
 	base::Timer _unmuteByFinishedTimer;
 
 	std::unordered_map<PeerId, std::unique_ptr<PeerData>> _peers;
-	std::unordered_map<PeerId, std::unique_ptr<History>> _histories;
 
 	MessageIdsList _mimeForwardIds;
 
@@ -989,6 +979,7 @@ private:
 	std::unique_ptr<CloudThemes> _cloudThemes;
 	std::unique_ptr<Streaming> _streaming;
 	std::unique_ptr<MediaRotation> _mediaRotation;
+	std::unique_ptr<Histories> _histories;
 	MsgId _nonHistoryEntryId = ServerMaxMsgId;
 
 	rpl::lifetime _lifetime;
